local entity = {}
entity_meta = {__index = entity}

function entity.new(x, y)
	local e = {
		pos = vec.new(x, y),
		vel = vec.new(),
		acc = vec.new(),
		dead = false,
		reachedGoal = false,
		hitObstacle = false,
		hitBorder = false,
		color = {math.random(), math.random(), math.random()},
		brain = {
			steps = {},
			currentStep = 1
		}
	}

	return setmetatable(e, entity_meta)
end

function entity:generateSteps(steps)
	for i=1, steps do
		local angle = (math.pi * 2) * math.random()
		self.brain.steps[i] = vec.new()
		self.brain.steps[i]:setLength(1)
		self.brain.steps[i]:setAngle(angle)
	end
end

function entity:update(dt)
	--Movement
	if self.brain.currentStep < #self.brain.steps and not self.dead then
		self.acc = self.brain.steps[self.brain.currentStep]
		self.brain.currentStep = self.brain.currentStep + 1
	else
		if not self.dead then
			OLD_AGE = OLD_AGE + 1
		end
		self:kill()
	end

	self.vel:add(self.acc, dt)
	self.vel:limit(5)
	self.pos:add(self.vel, dt)

	--Collision
	--border
	if self.pos.x < 0 or self.pos.x > width or self.pos.y < 0 or self.pos.y > height then
		if not self.dead then
			HIT_BORDER = HIT_BORDER + 1
		end
		self:kill()
		self.hitBorder = true
	end

	--goal
	if fmath.distance(self.pos.x, self.pos.y, goal.x, goal.y) < goal.radius then
		if not self.dead then
			REACHED_GOAL = REACHED_GOAL + 1
		end
		self:kill()
		self.reachedGoal = true
	end

	--Obstacle
	if fmath.distance(self.pos.x, self.pos.y, obstacle.x, obstacle.y) < obstacle.radius then
		if not self.dead then
			HIT_OBSTACLE = HIT_OBSTACLE + 1
		end
		self:kill()
		self.hitObstacle = true
	end
end

function entity:draw()
	local mode = "fill"
	love.graphics.setColor(self.color)
	if self.dead then
		love.graphics.setColor(1, 0, 0, 1)
		mode = "line"
	end
	love.graphics.circle(mode, self.pos.x, self.pos.y, 2)

	if self.dead then
		local mx, my = love.mouse.getPosition()
		if fmath.distance(self.pos.x, self.pos.y, mx, my) < 20 then
			love.graphics.print(self:fitness().."\n"..self.brain.currentStep, self.pos.x, self.pos.y - 40)
		end
	end
end

function entity:fitness()
	local fitness = 1000 - ((self.pos.x - goal.x)^2 + (self.pos.y - goal.y)^2)^0.5 - self.brain.currentStep

	if self.reachedGoal then
		fitness = fitness + 200
	end

	if self.hitObstacle or self.hitBorder then
		fitness = fitness - 200
	end

	if fitness < 1 then fitness = 1 end
	return fitness
	--((x2-x1)^2+(y2-y1)^2)^0.5
end

function entity:kill()
	self.dead = true
	self.acc:set(0, 0)
	self.vel:set(0, 0)
end

function entity:breed()
	local offspring = entity.new(POPULATION_POSITION.x, POPULATION_POSITION.y)
	offspring.brain.steps = deepcopy(self.brain.steps)
	return offspring
end

function entity:mutate()
	for i,v in ipairs(self.brain.steps) do
		if math.random() < MUTATION_CHANSE then
			local newAngle = v:getAngle()
			newAngle = newAngle + -1 + (math.random() * 2)
			v:setAngle(newAngle)
		end
	end
end

return entity













