local population = {}
local population_meta = {__index = population}

function population.new(size)
	local p = {
		list = {}
	}

	for i=1, size do
		local e = entity.new(POPULATION_POSITION.x, POPULATION_POSITION.y)
		e:generateSteps(STEPS)
		p.list[i] = e
	end

	return setmetatable(p, population_meta)
end

function population:update(dt)
	for i,v in ipairs(self.list) do
		v:update(dt)
	end
end

function population:draw()
	for i,v in ipairs(self.list) do
		v:draw()
	end
end	

function population:extinct()
	local extinct = true
	for i,v in ipairs(self.list) do
		if not v.dead then
			extinct = false
			break
		end
	end
	return extinct
end

function population:getFitnessSum()
	local sum = 0
	for i,v in ipairs(self.list) do
		sum = sum + v:fitness()
	end
	return sum
end

function population:nextGen()
	local newList = {}
	for i=1, POPULATION_SIZE do
		local parent = self:selectParent()
		newList[i] = parent:breed()
		newList[i]:mutate()
	end

	--STATS
	REACHED_GOAL = 0
	HIT_BORDER = 0
	HIT_OBSTACLE = 0
	OLD_AGE = 0

	self.list = newList
end

function population:selectParent()
	local fitnessSum = self:getFitnessSum()
	local r = math.random(fitnessSum)
	local rsum = 0
	for i,v in ipairs(self.list) do
		rsum = rsum + v:fitness()
		if rsum > r then
			return v
		end
	end
end

--[[

function population:select()
	local nextGen = {}

	local champ = self:getChamp()
	nextGen[1] = champ:breed()
	nextGen[1].champ = true
	
	for i=2, #self.list do
		local p = self:selectParent()
		nextGen[i] = p:breed()
		nextGen[i].brain:mutate()
	end


	self.list = nextGen
end]]

return population